<?php

namespace Drupal\media_download_all\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\File\FileSystem;
use Drupal\media_download_all\Plugin\Archiver\Zip;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Drupal\Core\Messenger\Messenger;

/**
 * Class DownloadController.
 */
class DownloadController extends ControllerBase {

  /**
   * Entity Field Manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * File system.
   *
   * @var \Drupal\Core\File\FileSystem
   */
  protected $fileSystem;

  /**
   * Provides messenger service.
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;

  /**
   * Constructs a new DownloadController object.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   * @param \Drupal\Core\File\FileSystem $file_system
   *   The file system.
   * @param \Drupal\Core\Messenger\Messenger
   *   The messenger service.
   */
  public function __construct(EntityFieldManagerInterface $entity_field_manager, FileSystem $file_system, Messenger $messenger) {
    $this->entityFieldManager = $entity_field_manager;
    $this->fileSystem = $file_system;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Symfony\Component\DependencyInjection\Exception\ServiceNotFoundException
   * @throws \Symfony\Component\DependencyInjection\Exception\ServiceCircularReferenceException
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_field.manager'),
      $container->get('file_system'),
      $container->get('messenger')
    );
  }

  /**
   * The main download method.
   *
   * @param string $entity_type
   *   The entity type.
   * @param int $entity_id
   *   The entity id.
   * @param string $field_name
   *   The field name.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   Return the file.
   *
   * @throws \InvalidArgumentException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Core\Archiver\ArchiverException
   */
  public function download($entity_type, $entity_id, $field_name) {

    list($media_ids, $label) = $this->getMediaIdsAndLabel($entity_type, $entity_id, $field_name);
    $file_ids = $this->getFileIds($media_ids);
    $zip_files_directory = "private://media_download_all";
    file_prepare_directory($zip_files_directory, FILE_CREATE_DIRECTORY);
    $file_path = $this->fileSystem->realpath($zip_files_directory) . "/$entity_type-$entity_id-$field_name.zip";
    $redirect_on_error_to = empty($_SERVER['HTTP_REFERER']) ? '/' : $_SERVER['HTTP_REFERER'];

    $file_zip = NULL;

    if (file_prepare_directory($zip_files_directory, FILE_CREATE_DIRECTORY)) {
      foreach ($file_ids as $fid) {
        if (!$file_zip instanceof Zip) {
          $file_zip = new Zip($file_path);
        }
        $file_zip->add($fid);
      }

      if ($file_zip instanceof Zip) {
        $file_zip->close();
        return $this->streamZipFile($file_path);
      }
      $this->messenger->addError($this->t('No files found for this entity to be downloaded'));
      return new RedirectResponse($redirect_on_error_to);
    }
    $this->messenger->addError($this->t('Zip file directory not found.'));
    return new RedirectResponse($redirect_on_error_to);
  }

  /**
   * Get Media IDs associated with the entity and its label.
   *
   * @param string $entity_type
   *   The entity type.
   * @param string $entity_id
   *   The entity ID.
   * @param string $field_name
   *   The field name.
   *
   * @return array
   *   The media IDs and the label of the entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   */
  protected function getMediaIdsAndLabel($entity_type, $entity_id, $field_name) {
    $entity_storage = $this->entityTypeManager()->getStorage($entity_type);
    $entity = $entity_storage->load($entity_id);
    if (NULL !== $entity) {
      $label = $entity->label();
    }
    $media_filed = $entity->{$field_name};
    $media_ids = [];

    foreach ($media_filed->getValue() as $item) {
      if (isset($item['target_id'])) {
        $media_ids[] = $item['target_id'];
      }
    }
    return [$media_ids, $label];
  }

  /**
   * Get file IDs.
   *
   * @param array $media_ids
   *   The media IDs.
   *
   * @return array
   *   The file IDs.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   */
  protected function getFileIds(array $media_ids) {
    $media_storage = $this->entityTypeManager()->getStorage('media');
    $medias = $media_storage->loadMultiple($media_ids);

    $file_ids = [];

    foreach ($medias as $media) {
      $bundle = $media->bundle();
      $file_field_names = $this->getFileFieldsOfBundle($bundle);
      foreach ($file_field_names as $file_field_name) {
        foreach ($media->{$file_field_name}->getValue() as $item) {
          if (isset($item['target_id'])) {
            $file_ids[] = $item['target_id'];
          }
        }
      }
    }

    return $file_ids;
  }

  /**
   * Get file field names.
   *
   * @param string $bundle
   *   The bundle of media.
   *
   * @return array
   *   Filed names contains file.
   */
  protected function getFileFieldsOfBundle($bundle) {

    $entityFieldManager = $this->entityFieldManager;

    $field_definitions = $entityFieldManager->getFieldDefinitions('media', $bundle);

    $field_names_filtered = [];
    foreach ($field_definitions as $field_name => $field_definition) {
      if ($field_name !== "thumbnail" && $field_definition->getFieldStorageDefinition()->getSetting('target_type') === 'file') {
        $field_names_filtered[] = $field_name;
      }
    }

    return $field_names_filtered;

  }

  /**
   * Method to stream created zip file.
   *
   * @param string $file_path
   *   File physical path.
   *
   * @return \Symfony\Component\HttpFoundation\BinaryFileResponse
   *   The binary file.
   */
  protected function streamZipFile($file_path) {
    $binary_file_response = new BinaryFileResponse($file_path);
    $binary_file_response->setContentDisposition(ResponseHeaderBag::DISPOSITION_ATTACHMENT, basename($file_path));
    return $binary_file_response;
  }

}
